@@ -6,6 +6,7 @@ from typing import Any
 
 from .common import (
     AirzoneStages,
+    GrilleAngle,
     OperationMode,
     SystemType,
     TemperatureUnit,
@@ -13,6 +14,7 @@ from .common import (
 )
 from .const import (
     API_AIR_DEMAND,
+    API_COLD_ANGLE,
     API_COLD_STAGE,
     API_COLD_STAGES,
     API_COOL_MAX_TEMP,
@@ -23,13 +25,16 @@ from .const import (
     API_ERROR_LOW_BATTERY,
     API_ERRORS,
     API_FLOOR_DEMAND,
+    API_HEAT_ANGLE,
     API_HEAT_MAX_TEMP,
     API_HEAT_MIN_TEMP,
     API_HEAT_SET_POINT,
     API_HEAT_STAGE,
     API_HEAT_STAGES,
     API_HUMIDITY,
+    API_MANUFACTURER,
     API_MAX_TEMP,
+    API_MC_CONNECTED,
     API_MIN_TEMP,
     API_MODE,
     API_MODES,
@@ -38,11 +43,13 @@ from .const import (
     API_POWER,
     API_ROOM_TEMP,
     API_SET_POINT,
+    API_SLEEP,
     API_SPEED,
     API_SPEEDS,
     API_SYSTEM_FIRMWARE,
     API_SYSTEM_ID,
     API_SYSTEM_TYPE,
+    API_TEMP_STEP,
     API_THERMOS_FIRMWARE,
     API_THERMOS_RADIO,
     API_THERMOS_TYPE,
@@ -50,6 +57,8 @@ from .const import (
     API_ZONE_ID,
     AZD_AIR_DEMAND,
     AZD_BATTERY_LOW,
+    AZD_CLAMP_METER,
+    AZD_COLD_ANGLE,
     AZD_COLD_STAGE,
     AZD_COLD_STAGES,
     AZD_COOL_TEMP_MAX,
@@ -62,6 +71,7 @@ from .const import (
     AZD_FIRMWARE,
     AZD_FLOOR_DEMAND,
     AZD_FULL_NAME,
+    AZD_HEAT_ANGLE,
     AZD_HEAT_STAGE,
     AZD_HEAT_STAGES,
     AZD_HEAT_TEMP_MAX,
@@ -69,6 +79,7 @@ from .const import (
     AZD_HEAT_TEMP_SET,
     AZD_HUMIDITY,
     AZD_ID,
+    AZD_MANUFACTURER,
     AZD_MASTER,
     AZD_MODE,
     AZD_MODEL,
@@ -76,6 +87,7 @@ from .const import (
     AZD_NAME,
     AZD_ON,
     AZD_PROBLEMS,
+    AZD_SLEEP,
     AZD_SPEED,
     AZD_SPEEDS,
     AZD_SYSTEM,
@@ -83,6 +95,7 @@ from .const import (
     AZD_TEMP_MAX,
     AZD_TEMP_MIN,
     AZD_TEMP_SET,
+    AZD_TEMP_STEP,
     AZD_TEMP_UNIT,
     AZD_THERMOSTAT_FW,
     AZD_THERMOSTAT_MODEL,
@@ -101,12 +114,15 @@ _LOGGER = logging.getLogger(__name__)
 class System:
     """Airzone System."""
 
-    def __init__(self, airzone_system):
+    def __init__(self, airzone_system: list[dict[str, Any]]):
         """System init."""
+        self.clamp_meter: bool | None = None
         self.energy: int | None = None
         self.errors: list[str] = []
-        self.id = None
+        self.id: int | None = None
         self.firmware: str | None = None
+        self.manufacturer: str | None = None
+        self.mode: OperationMode | None = None
         self.modes: list[OperationMode] = []
         self.type: SystemType | None = None
         self.zones: dict[int, Zone] = {}
@@ -130,9 +146,14 @@ class System:
             AZD_ZONES_NUM: self.num_zones(),
         }
 
-        energy = self.get_energy()
-        if energy is not None:
-            data[AZD_ENERGY] = energy
+        clamp_meter = self.get_clamp_meter()
+        if clamp_meter is not None:
+            data[AZD_CLAMP_METER] = clamp_meter
+
+            if clamp_meter:
+                energy = self.get_energy()
+                if energy is not None:
+                    data[AZD_ENERGY] = energy
 
         errors = self.get_errors()
         if len(errors) > 0:
@@ -146,10 +167,22 @@ class System:
         if full_name is not None:
             data[AZD_FULL_NAME] = full_name
 
+        manufacturer = self.get_manufacturer()
+        if manufacturer is not None:
+            data[AZD_MANUFACTURER] = manufacturer
+
+        mode = self.get_mode()
+        if mode is not None:
+            data[AZD_MODE] = mode
+
         model = self.get_model()
         if self.type is not None:
             data[AZD_MODEL] = model
 
+        modes = self.get_modes()
+        if modes is not None:
+            data[AZD_MODES] = modes
+
         return data
 
     def add_error(self, val: str) -> None:
@@ -157,6 +190,10 @@ class System:
         if val not in self.errors:
             self.errors.append(val)
 
+    def get_clamp_meter(self) -> bool | None:
+        """Return system clamp meter connection."""
+        return self.clamp_meter
+
     def get_energy(self) -> int | None:
         """Return system energy consumption."""
         return self.energy
@@ -165,7 +202,7 @@ class System:
         """Return system errors."""
         return self.errors
 
-    def get_id(self) -> int:
+    def get_id(self) -> int | None:
         """Return system ID."""
         return self.id
 
@@ -179,15 +216,27 @@ class System:
         """Return full name."""
         return f"Airzone [{self.get_id()}] System"
 
+    def get_manufacturer(self) -> str | None:
+        """Return system manufacturer."""
+        return self.manufacturer
+
     def get_model(self) -> str | None:
         """Return system model."""
         if self.type:
             return str(self.type)
         return None
 
-    def get_modes(self) -> list[OperationMode]:
+    def get_mode(self) -> OperationMode | None:
+        """Return system mode."""
+        return self.mode
+
+    def get_modes(self) -> list[OperationMode] | None:
         """Return system modes."""
-        return self.modes
+        if len(self.modes) > 0:
+            return self.modes
+        if self.mode is not None:
+            return [self.mode]
+        return None
 
     def get_problems(self) -> bool:
         """Return system problems."""
@@ -198,24 +247,43 @@ class System:
         for zone in self.zones.values():
             if zone.get_id() == zone_id:
                 return zone
-        raise InvalidZone
+        raise InvalidZone(f"Zone {zone_id} not present in System {self.id}")
 
     def num_zones(self) -> int:
         """Return number of system zones."""
         return len(self.zones)
 
+    def set_mode(self, mode: OperationMode) -> None:
+        """Set system mode."""
+        self.mode = mode
+
     def set_modes(self, modes: list[OperationMode]) -> None:
         """Set system modes."""
         self.modes = modes
 
     def set_param(self, key: str, value: Any) -> None:
-        """Update zones parameters by key and value."""
+        """Update parameters by key and value."""
+        if key == API_MODE:
+            self.mode = OperationMode(value)
+
         for zone in self.zones.values():
             zone.set_param(key, value)
 
     def update_data(self, data: dict[str, Any]) -> None:
         """Update system parameters by dict."""
 
+        if API_MC_CONNECTED in data:
+            self.clamp_meter = bool(data[API_MC_CONNECTED])
+
+        if API_ERRORS in data:
+            errors: list[dict[str, str]] = data[API_ERRORS]
+            for error in errors:
+                for val in error.values():
+                    self.add_error(val)
+
+        if API_MANUFACTURER in data:
+            self.manufacturer = str(data[API_MANUFACTURER])
+
         if API_POWER in data:
             self.energy = int(data[API_POWER])
 
@@ -269,6 +337,7 @@ class Zone:
     def __init__(self, system: System, zone: dict[str, Any]):
         """Zone init."""
         self.air_demand = bool(zone[API_AIR_DEMAND])
+        self.cold_angle: GrilleAngle | None = None
         self.cold_stage: AirzoneStages | None = None
         self.cold_stages: list[AirzoneStages] = []
         self.cool_temp_max: float | None = None
@@ -277,6 +346,7 @@ class Zone:
         self.double_set_point: bool = False
         self.errors: list[str] = []
         self.floor_demand = bool(zone[API_FLOOR_DEMAND])
+        self.heat_angle: GrilleAngle | None = None
         self.heat_temp_max: float | None = None
         self.heat_temp_min: float | None = None
         self.heat_temp_set: float | None = None
@@ -289,12 +359,14 @@ class Zone:
         self.modes: list[OperationMode] = []
         self.name = str(zone[API_NAME])
         self.on = bool(zone[API_ON])
+        self.sleep: int | None = None
         self.speed: int | None = None
-        self.speeds: int | None = None
+        self.speeds: list[int] = []
         self.temp = float(zone[API_ROOM_TEMP])
         self.temp_max = float(zone[API_MAX_TEMP])
         self.temp_min = float(zone[API_MIN_TEMP])
         self.temp_set = float(zone[API_SET_POINT])
+        self.temp_step: float | None = None
         self.temp_unit = TemperatureUnit(zone[API_UNITS])
         self.thermostat = Thermostat(zone)
         self.system = system
@@ -302,6 +374,11 @@ class Zone:
         if API_HUMIDITY in zone:
             self.humidity = int(zone[API_HUMIDITY])
 
+        if API_COLD_ANGLE in zone:
+            self.cold_angle = GrilleAngle(zone[API_COLD_ANGLE])
+        if API_HEAT_ANGLE in zone:
+            self.heat_angle = GrilleAngle(zone[API_HEAT_ANGLE])
+
         if API_COLD_STAGE in zone:
             self.cold_stage = AirzoneStages(zone[API_COLD_STAGE])
         if API_COLD_STAGES in zone:
@@ -341,17 +418,27 @@ class Zone:
                 for key, val in error.items():
                     self.add_error(key, val)
 
+        if API_SLEEP in zone:
+            self.sleep = int(zone[API_SLEEP])
+
         if API_SPEED in zone:
             self.speed = int(zone[API_SPEED])
         if API_SPEEDS in zone:
-            self.speeds = int(zone[API_SPEEDS])
+            speeds = int(zone[API_SPEEDS])
+            self.speeds = list(range(0, speeds + 1))
+
+        if API_TEMP_STEP in zone:
+            self.temp_step = float(zone[API_TEMP_STEP])
 
         if self.master:
             for mode in zone[API_MODES]:
                 self.modes.append(OperationMode(mode))
+            self.system.set_mode(self.mode)
+            self.system.set_modes(self.modes)
             if OperationMode.STOP not in self.modes:
                 self.modes.append(OperationMode.STOP)
-            self.system.set_modes(self.modes)
+        elif self.system.get_mode() is None:
+            self.system.set_mode(self.mode)
 
     def data(self) -> dict[str, Any]:
         """Return Airzone zone data."""
@@ -408,6 +495,13 @@ class Zone:
             if heat_temp_set:
                 data[AZD_HEAT_TEMP_SET] = heat_temp_set
 
+        cold_angle = self.get_cold_angle()
+        if cold_angle is not None:
+            data[AZD_COLD_ANGLE] = cold_angle
+        heat_angle = self.get_heat_angle()
+        if heat_angle is not None:
+            data[AZD_HEAT_ANGLE] = heat_angle
+
         cold_stage = self.get_cold_stage()
         if cold_stage is not None:
             data[AZD_COLD_STAGE] = cold_stage
@@ -422,6 +516,10 @@ class Zone:
         if heat_stages is not None:
             data[AZD_HEAT_STAGES] = heat_stages
 
+        sleep = self.get_sleep()
+        if sleep is not None:
+            data[AZD_SLEEP] = sleep
+
         speed = self.get_speed()
         if speed is not None:
             data[AZD_SPEED] = speed
@@ -437,6 +535,10 @@ class Zone:
         if modes is not None:
             data[AZD_MODES] = modes
 
+        temp_step = self.get_temp_step()
+        if temp_step is not None:
+            data[AZD_TEMP_STEP] = temp_step
+
         thermostat_firmware = self.thermostat.get_firmware()
         if thermostat_firmware is not None:
             data[AZD_THERMOSTAT_FW] = thermostat_firmware
@@ -474,6 +576,10 @@ class Zone:
             return API_ERROR_LOW_BATTERY in self.errors
         return None
 
+    def get_cold_angle(self) -> GrilleAngle | None:
+        """Return zone cold angle."""
+        return self.cold_angle
+
     def get_cold_stage(self) -> AirzoneStages | None:
         """Return zone cold stage."""
         return self.cold_stage
@@ -528,6 +634,10 @@ class Zone:
         """Return zone ID."""
         return self.id
 
+    def get_heat_angle(self) -> GrilleAngle | None:
+        """Return zone heat angle."""
+        return self.heat_angle
+
     def get_heat_stage(self) -> AirzoneStages | None:
         """Return zone heat stage."""
         return self.heat_stage
@@ -575,10 +685,10 @@ class Zone:
         if self.master:
             return self.modes
         modes = self.system.get_modes()
-        if len(modes) == 0:
+        if modes is None:
             modes = [self.mode]
-            if OperationMode.STOP not in modes:
-                modes.append(OperationMode.STOP)
+        if OperationMode.STOP not in modes:
+            modes.append(OperationMode.STOP)
         return modes
 
     def get_name(self) -> str:
@@ -593,15 +703,21 @@ class Zone:
         """Return zone problems."""
         return bool(self.errors)
 
+    def get_sleep(self) -> int | None:
+        """Return zone sleep time in minutes."""
+        return self.sleep
+
     def get_speed(self) -> int | None:
         """Return zone speed."""
         return self.speed
 
-    def get_speeds(self) -> int | None:
+    def get_speeds(self) -> list[int] | None:
         """Return zone speeds."""
-        return self.speeds
+        if len(self.speeds) > 0:
+            return self.speeds
+        return None
 
-    def get_system_id(self) -> int:
+    def get_system_id(self) -> int | None:
         """Return system ID."""
         return self.system.get_id()
 
@@ -625,6 +741,12 @@ class Zone:
         """Return zone set temperature."""
         return round(self.temp_set, 1)
 
+    def get_temp_step(self) -> float | None:
+        """Return zone step temperature."""
+        if self.temp_step is not None:
+            return round(self.temp_step, 1)
+        return None
+
     def get_temp_unit(self) -> TemperatureUnit:
         """Return zone temperature unit."""
         return self.temp_unit
