@@ -17,6 +17,7 @@ from .const import (
     API_ERROR_SYSTEM_ID_NOT_AVAILABLE,
     API_ERROR_SYSTEM_ID_OUT_RANGE,
     API_ERROR_ZONE_ID_NOT_AVAILABLE,
+    API_ERROR_ZONE_ID_NOT_PROVIDED,
     API_ERROR_ZONE_ID_OUT_RANGE,
     API_ERRORS,
     API_HVAC,
@@ -36,6 +37,9 @@ from .const import (
     DEFAULT_PORT,
     DEFAULT_SYSTEM_ID,
     HTTP_CALL_TIMEOUT,
+    RAW_HVAC,
+    RAW_SYSTEMS,
+    RAW_WEBSERVER,
 )
 from .device import System, Zone
 from .exceptions import (
@@ -50,6 +54,7 @@ from .exceptions import (
     SystemNotAvailable,
     SystemOutOfRange,
     ZoneNotAvailable,
+    ZoneNotProvided,
     ZoneOutOfRange,
 )
 from .webserver import WebServer
@@ -83,6 +88,7 @@ class AirzoneLocalApi:
         options: ConnectionOptions,
     ):
         """Device init."""
+        self._api_raw_data: dict[str, Any] = {}
         self.aiohttp_session = aiohttp_session
         self.api_features: int = ApiFeature.HVAC
         self.api_features_checked = False
@@ -94,20 +100,22 @@ class AirzoneLocalApi:
     def handle_errors(errors: list[dict[str, str]]) -> None:
         """Handle API errors."""
         for error in errors:
-            for val in error.values():
+            for key, val in error.items():
                 if val == API_ERROR_METHOD_NOT_SUPPORTED:
-                    raise InvalidMethod
+                    raise InvalidMethod(f"{key}: {val}")
                 if val == API_ERROR_REQUEST_MALFORMED:
-                    raise RequestMalformed
+                    raise RequestMalformed(f"{key}: {val}")
                 if val == API_ERROR_SYSTEM_ID_NOT_AVAILABLE:
-                    raise SystemNotAvailable
+                    raise SystemNotAvailable(f"{key}: {val}")
                 if val == API_ERROR_SYSTEM_ID_OUT_RANGE:
-                    raise SystemOutOfRange
+                    raise SystemOutOfRange(f"{key}: {val}")
                 if val == API_ERROR_ZONE_ID_OUT_RANGE:
-                    raise ZoneOutOfRange
+                    raise ZoneOutOfRange(f"{key}: {val}")
                 if val == API_ERROR_ZONE_ID_NOT_AVAILABLE:
-                    raise ZoneNotAvailable
-                raise APIError(error)
+                    raise ZoneNotAvailable(f"{key}: {val}")
+                if val == API_ERROR_ZONE_ID_NOT_PROVIDED:
+                    raise ZoneNotProvided(f"{key}: {val}")
+                raise APIError(f"{key}: {val}")
 
     async def http_request(
         self, method: str, path: str, data: Any | None = None
@@ -126,16 +134,16 @@ class AirzoneLocalApi:
             if API_ERRORS in resp_json:
                 self.handle_errors(resp_json[API_ERRORS])
             raise APIError(f"HTTP status: {resp.status}")
-        return cast(dict, resp_json)
+        return cast(dict[str, Any], resp_json)
 
-    def update_systems(self, data) -> None:
+    def update_systems(self, data: dict[str, Any]) -> None:
         """Gather Systems data."""
         for api_system in data[API_SYSTEMS]:
             system = self.get_system(api_system[API_SYSTEM_ID])
             if system:
                 system.update_data(api_system)
 
-    def update_webserver(self, data) -> None:
+    def update_webserver(self, data: dict[str, Any]) -> None:
         """Gather WebServer data."""
         self.webserver = WebServer(data)
 
@@ -156,7 +164,7 @@ class AirzoneLocalApi:
                 self.api_features |= ApiFeature.SYSTEMS
                 if update:
                     self.update_systems(systems)
-        except SystemOutOfRange:
+        except (SystemOutOfRange, ZoneNotProvided):
             pass
 
         self.api_features_checked = True
@@ -177,11 +185,11 @@ class AirzoneLocalApi:
         await self.check_features(False)
 
         response = await self.get_hvac()
-        if self.options.system_id == 0:
+        if self.options.system_id == DEFAULT_SYSTEM_ID:
             if API_SYSTEMS not in response:
-                raise InvalidHost
+                raise InvalidHost(f"validate: {API_SYSTEMS} not in API response")
         elif API_DATA not in response:
-            raise InvalidHost
+            raise InvalidHost(f"validate: {API_DATA} not in API response")
 
         if self.webserver:
             return self.webserver.get_mac()
@@ -193,22 +201,24 @@ class AirzoneLocalApi:
         systems: dict[int, System] = {}
 
         hvac = await self.get_hvac()
-        if self.options.system_id == 0:
+        if self.options.system_id == DEFAULT_SYSTEM_ID:
             for hvac_system in hvac[API_SYSTEMS]:
                 system = System(hvac_system[API_DATA])
-                if system:
-                    systems[system.get_id()] = system
+                if system and (_id := system.get_id()):
+                    systems[_id] = system
         else:
             system = System(hvac[API_DATA])
-            if system:
-                systems[system.get_id()] = system
+            if system and (_id := system.get_id()):
+                systems[_id] = system
         self.systems = systems
 
         await self.update_features()
 
         return bool(systems)
 
-    async def get_hvac_systems(self, params: dict[str, Any] = None) -> dict[str, Any]:
+    async def get_hvac_systems(
+        self, params: dict[str, Any] | None = None
+    ) -> dict[str, Any]:
         """Return Airzone HVAC systems."""
         if not params:
             params = {
@@ -219,9 +229,10 @@ class AirzoneLocalApi:
             f"{API_V1}/{API_HVAC}",
             params,
         )
+        self._api_raw_data[RAW_SYSTEMS] = res
         return res
 
-    async def get_hvac(self, params: dict[str, Any] = None) -> dict[str, Any]:
+    async def get_hvac(self, params: dict[str, Any] | None = None) -> dict[str, Any]:
         """Return Airzone HVAC zones."""
         if not params:
             params = {
@@ -233,6 +244,7 @@ class AirzoneLocalApi:
             f"{API_V1}/{API_HVAC}",
             params,
         )
+        self._api_raw_data[RAW_HVAC] = res
         return res
 
     async def get_webserver(self) -> dict[str, Any]:
@@ -241,31 +253,42 @@ class AirzoneLocalApi:
             "POST",
             f"{API_V1}/{API_WEBSERVER}",
         )
+        self._api_raw_data[RAW_WEBSERVER] = res
         return res
 
     async def put_hvac(self, params: dict[str, Any]) -> dict[str, Any]:
-        """Return Airzone HVAC."""
-        res = await self.http_request(
+        """Perform a PUT request to update HVAC parameters."""
+        return await self.http_request(
             "PUT",
             f"{API_V1}/{API_HVAC}",
             params,
         )
 
+    async def set_hvac_parameters(self, params: dict[str, Any]) -> dict[str, Any]:
+        """Set Airzone HVAC parameters and handle response."""
+        res = await self.put_hvac(params)
+
         if API_DATA not in res:
             if API_ERRORS in res:
                 self.handle_errors(res[API_ERRORS])
-            raise APIError(f"{API_DATA} not in API response")
+            raise APIError(f"set_hvac: {API_DATA} not in API response")
 
-        data: dict = res[API_DATA][0]
+        data: dict[str, Any] = res[API_DATA][0]
         for key, value in params.items():
             if key not in data or data[key] != value:
                 if key == API_SYSTEM_ID and value != 0:
-                    raise InvalidSystem
+                    raise InvalidSystem(
+                        f"set_hvac: System mismatch: {data.get(key)} vs {value}"
+                    )
                 if key == API_ZONE_ID and value != 0:
-                    raise InvalidZone
+                    raise InvalidZone(
+                        f"set_hvac: Zone mismatch: {data.get(key)} vs {value}"
+                    )
                 if key not in data:
-                    raise InvalidParam
-                raise ParamUpdateFailure
+                    raise InvalidParam(f"set_hvac: param not in data: {key}={value}")
+                raise ParamUpdateFailure(
+                    f"set_hvac: param update failure: {key}={value}"
+                )
 
         system = self.get_system(data[API_SYSTEM_ID])
         zone = self.get_zone(data[API_SYSTEM_ID], data[API_ZONE_ID])
@@ -277,6 +300,10 @@ class AirzoneLocalApi:
 
         return res
 
+    def raw_data(self) -> dict[str, Any]:
+        """Return raw Airzone API data."""
+        return self._api_raw_data
+
     def data(self) -> dict[str, Any]:
         """Return Airzone device data."""
         data: dict[str, Any] = {
@@ -303,7 +330,7 @@ class AirzoneLocalApi:
         for system in self.systems.values():
             if system.get_id() == system_id:
                 return system
-        raise InvalidSystem
+        raise InvalidSystem(f"System {system_id} not present")
 
     def get_zone(self, system_id: int, zone_id: int) -> Zone:
         """Return Airzone system zone."""
